#!/usr/bin/env python3

import urllib.parse, urllib.request
import json

SETTINGS = {
  'url': 'http://query.yahooapis.com/v1/public/yql',
  'db_url': 'store://datatables.org/alltableswithkeys',
  'stocks': 'yahoo.finance.quotes',
  'option': 'yahoo.finance.options'
}

class YQLQuery(object):
  def __init__(self, s):
    self.params = {
      'symbol': s,
    }

  def get_stock(self, **kwargs):
    self.__format_args(kwargs)
    columns = self.__format_column_list(self.params['columns'])
    yql = 'select {} from {} where symbol = \'{}\''.format( \
      columns, SETTINGS['stocks'], self.params['symbol'])
    response = self.execute(yql)
    return self.__validate_response(response, 'quote')

  def get_option(self, **kwargs):
    self.__format_args(kwargs)
    columns = self.__format_column_list(self.params['columns'])
    yql = 'select {} from {} where symbol = \'{}\''.format( \
      columns, SETTINGS['option'], self.params['symbol'])
    if not 'exp' in self.params:
      raise SyntaxError("Expiration argument is required")
    yql += " and expiration='{}'".format(self.params['exp'])
    response = self.execute(yql)
    return self.__validate_response(response, 'optionsChain')

  def get_recommendation(self):
    yql = 'select * from html where url="http://finance.yahoo.com/q/ao?s={}&ql=1"' \
          'and xpath=\'//table[@class="yfnc_datamodoutline1 equaltable"]/tr[2]/td[2][@class="yfnc_tabledata1"]/p\''.format(
            self.params['symbol'])
    response = self.execute(yql)
    return {'Recommendation': response['query']['results']['p'][0]}

  def execute(self, yql):
    queryString = urllib.parse.urlencode( \
      {'q': yql, 'format': 'json', 'env': SETTINGS['db_url']})
    with urllib.request.urlopen(SETTINGS['url']+'?'+queryString) as s:
      return json.loads(bytes.decode(s.read()))

  def __format_args(self,args):
    for k,v in args.items():
      if k not in self.params:
        self.params[k] = v
    if not 'columns' in self.params:
      self.params['columns'] = '*'

  def __format_column_list(self, columns):
    return ','.join([column for column in columns])

  def __is_valid_response(self, response, field):
    return 'query' in response and 'results' in response['query'] \
           and field in response['query']['results']

  def __validate_response(self, response, tagToCheck):
    if self.__is_valid_response(response, tagToCheck):
      quoteInfo = response['query']['results'][tagToCheck]
    else:
      if 'error' in response:
        raise QueryError(
          'YQL query failed with error: "{0}"'.format( \
            response['error']['description']))
      else:
        raise QueryError('YQL response malformed.')
    return quoteInfo

class QueryError(Exception):
  def __init__(self, value):
    self.value = value

  def __str__(self):
    return repr(self.value)

if __name__ == '__main__':
  for o in ('msft','goog','yhoo'):
    x = YQLQuery(o)
    print(o)
    print(x.get_stock())
    print(x.get_stock(columns=('symbol','MarketCapitalization')))
    print(x.get_option(exp='2013-01'))
    print(x.get_option(exp='2013-01',columns=('option.strikePrice','option.lastPrice','symbol')))
    print(x.get_recommendation())
    print('\n')
